/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    apisnmp.c
 *
 * Date:        May 1994
 *
 * Purpose:     This file implements the interface to the SNMP environment.
 *
 * Author(s):   J.W.Togtema, Eric van Hengstum & Vincent Berkhout
 *		Antoinne Olle'man
 *
 * -----------------------------------------------------------------------
 */

#include <apiMSG.h>
#include <sim.h>
#include <apiSNMP.h>

/*
 * 	local variables
 */
static	IADDRESS	myAddress;	/* local address of application */
static 	IADDRESS	spmAddress="spm" ; 
static	MSGTABLE	msgCache ;
static	int		msgCacheIndex ;

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpOpen
 *
 * Arguments:	localAddress, reference to local application (IN).
 *		role, reference to desired role (IN).
 *
 * Purpose: 	This function initialises this entity. It does so by following
 *		thoose steps:
 *		1) Start the SPM, but just in situations the SPM is not 
 *		   already running.
 *			ERROR ->	Function returns ERROR.
 *		2) Open connection to SPM.
 *			ERROR ->	Function returns ERROR.
 *		3) Send indentification message to SPM
 *			ERROR ->	Function returns ERROR.
 *		4) Function returns OK
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
snmpOpen( NAME application, char **arge )	
{
       	pid_t pid;
	MSG *mMsg;
	register short  i = 0;
	AsnInt mHandle;
	SPTYPE mType=AGENTANDMAN_ADM;
	IADDRESS localAddress;


	InitNibbleMem( 64*1024, 512);

	memcpy( localAddress, application, 256 );
	for( i = 0; i < MSGTABLEMAX; i++ )
		msgCache[i].type = DEFAULTVALUE;
	msgCacheIndex = 0;

	(void) strcpy( myAddress, localAddress );
	if( (pid = fork( )) == (pid_t) -1 )
		return( ERROR );
	
	if( pid == (pid_t) 0 ) {
		/* (void) system( spmAddress ); */
		execve ( "spm", NULL, arge );
		exit( OK ); 
	}
	(void) sleep( 8 ); /* happy hacking */
	if( simOpen( myAddress, (char *) spmAddress ) != OK ) {
		(void) simClose( (char *) spmAddress );
		return( ERROR );
	}
	if( msgOpen( &mHandle, &mMsg ) != OK ) {
		return( ERROR );
	}
	(void) msgPut( mType, NULL, (QOS) NULL, NULL, (AsnInt) NULL, NULL, (AsnInt) NULL, 
			(AsnInt) NULL, mHandle );

	if( snmpSend( spmAddress, mMsg ) != OK ) {
		(void) msgClose( mHandle );
		return( ERROR );
	}
	(void) msgClose( mHandle );
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpLook
 *
 * Arguments:	mType, reference to messagetype (OUT).
 *
 * Purpose: 	This function retrieves the type of the message with as sender
 *		SPM, which is pending to be received by the local application.
 *		Steps:
 *		1) Receive all messages from SPM and put them in bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Retrieve type of first message.
 *			ERROR ->	Function returns ERROR.
 *		3) Function returns OK.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpLook( SPTYPE *eventType )
{
	AsnInt mHandle;
	MSG *mMsg;
	int s;
	char m[32*1024];
	IADDRESS remoteAddress;

	*eventType = 0;
	if( msgOpen( &mHandle, &mMsg ) != OK ) {
		return( ERROR );
	}
	while( simReceive( (char *) remoteAddress, (char *) m ) == OK ) {
		memcpy( &s, m, 4 );
		memcpy( mMsg, m+4, s );
		if( msgOpen( &mHandle, &mMsg ) != OK ) {
			return( ERROR );
		}
	}
	(void) msgClose( mHandle );

	if( msgCache[msgCacheIndex].type != DEFAULTVALUE ) {
		if( snmpSend( spmAddress, &msgCache[msgCacheIndex] ) == OK ) {
			msgCache[msgCacheIndex].type = DEFAULTVALUE;
			if( ++msgCacheIndex == MSGTABLEMAX )
				msgCacheIndex = 0;
		}
	} else
		if( ++msgCacheIndex == MSGTABLEMAX )
				msgCacheIndex = 0;

	*eventType = 0;
	if( msgGet( eventType, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
		    &mHandle ) != OK ) {
		return( ERROR );
	}
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpClose
 *
 * Arguments:	how, specifies the closing manner. (IN)
 *
 * Purpose: 	This function closes the entity.
 *		Steps:
 *		1) Clear all active buffers.
 *		2) Close connection to SPM.
 *			ERROR ->	Function returns ERROR.
 *		3) Function returns OK.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpClose( HOW how )
{
	short i = (short) how;
	AsnInt mHandle;
	SPTYPE mType=AGENTANDMAN_ADM;
	MSG *mMsg;


	if( msgOpen( &mHandle, &mMsg ) != OK ) {
		return( ERROR );
	}
	(void) msgPut( mType, NULL, (QOS) NULL, NULL, (AsnInt) NULL, NULL, 
			(AsnInt) NULL, (AsnInt) NULL, mHandle );
		       
	if( snmpSend( spmAddress, mMsg ) !=  OK ) {
		return( ERROR );
	}
	(void) msgClose( mHandle ); 
	(void) sleep( 2 );
	for( i = 0; i < MSGTABLEMAX; i++ )
		(void) msgClose( i );
	simClose( (char *) spmAddress );
	ShutdownNibbleMem( );

	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpErrorDescryption
 *
 * Arguments:	mReqId, identifies erroneous message (OUT).
 *		mErrType, specifies errortype (OUT).
 *
 * Purpose: 	This function provides additional information on erroneous
 *		message.
 *		Steps:
 *		1) Retrieve erroneous message from bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Release messagebuffer.
 *		4) Function returns OK.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpErrorDescription( INTEGER *requestId, 
		      DESCRIPTION error )
{
#define max_snmp_errors 9
	char * snmpapi_errors[ max_snmp_errors ] = {
		/* 00 */ "Unknown error.", 
		/* 01 */ "Request timed out.",
		/* 02 */ "No such context available.",
		/* 03 */ "Unknown host.", 
		/* 04 */ "No correct ACL entry found.",
		/* 05 */ "No such transport domain.",
		/* 06 */ "Error in construction of packet.", 
		/* 07 */ "Too many variable bindings."
		/* 08 */ "Unknown quality of service."
	}; 
	AsnInt mHandle, mErrType;
	SPTYPE mType = ERR_IND;

	if( msgGet( &mType, NULL, NULL, NULL, requestId, NULL, &mErrType, NULL,
		    &mHandle ) != OK )
		return FAILED;
	(void) msgClose( mHandle );
	strcpy( error, snmpapi_errors[ mErrType ] );
	return( OK );
}



/* 
 *		Support routines start here.
 */



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpReceiveIndication
 *
 * Arguments:	mType, specifies messagetype (IN).
 *		mReqId, identifies message (OUT).
 *		mErrInd, specifies erroneous variable (OUT)
 *		mErrStat, identifies errortype (OUT).
 *		mConName, specifies contextname (OUT).
 *		mVarBinds, reference to variablebindingslist (OUT).
 *
 * Purpose: 	This function retrieves the information contained in the 
 *		mType message.
 *		Steps:
 *		1) Retrieve message from bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Release messagebuffer.
 *		3) Function returns OK
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpReceiveIndication( SPTYPE mType, INTEGER *mReqId, INTEGER *mErrInd,
		       INTEGER *mErrStat, char *mConName, VARLIST **mVarBinds )
{
	AsnInt mHandle;

	if( msgGet( &mType, NULL, NULL, mConName, mReqId, *mVarBinds, 
		    mErrStat, mErrInd, &mHandle ) != OK )
		return( ERROR );
	(void) msgClose( mHandle );
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpReceiveNotification
 *
 * Arguments:	mType, specifies messagetype (IN).
 *		mSysName, specifies systemname (OUT).
 *		mVarBinds, reference to variablebindingslist (OUT).
 *
 * Purpose: 	This function retrieves the information contained in the 
 *		mType message.
 *		Steps:
 *		1) Retrieve message from bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Release messagebuffer.
 *		3) Function returns OK
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpReceiveNotification( SPTYPE mType, char *mSysName, char *mConName, 
		   	 VARLIST **mVarBinds )
{
	AsnInt mHandle;

	if( msgGet( &mType, mSysName, NULL, mConName, NULL, *mVarBinds, NULL,
		    NULL, &mHandle ) != OK )
		return( ERROR );
	(void) msgClose( mHandle );
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpReceiveConfirm
 *
 * Arguments:	mType, specifies messagetype (IN).
 *		mReqId, identifies message (OUT).
 *		mErrStat, identifies errortype (OUT).
 *		mErrInd, specifies erroneous variable (OUT)
 *		mVarBinds, reference to variablebindingslist (OUT).
 *
 * Purpose: 	This function retrieves the information contained in the 
 *		mType message.
 *		Steps:
 *		1) Retrieve message from bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Release messagebuffer.
 *		3) Function returns OK
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpReceiveConfirm( SPTYPE mType, INTEGER *mReqId, INTEGER *mErrStat, 
		    INTEGER *mErrInd, VARLIST **mVarBinds )
{
	AsnInt mHandle;

	if( msgGet( &mType, NULL, NULL, NULL, mReqId, *mVarBinds, mErrStat,
		    mErrInd, &mHandle ) != OK )
		return( ERROR );
	(void) msgClose( mHandle );
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpReceiveNegConfirm
 *
 * Arguments:	mType, specifies messagetype (IN).
 *		mReqId, identifies message (OUT).
 *
 * Purpose: 	This function retrieves the information contained in the 
 *		mType message.
 *		Steps:
 *		1) Retrieve message from bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Release messagebuffer.
 *		3) Function returns OK
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpReceiveNegConfirm( SPTYPE mType, INTEGER *mReqId )
{
	AsnInt mHandle;

	if( msgGet( &mType, NULL, NULL, NULL, mReqId, NULL, NULL, NULL, 
		    &mHandle ) != OK )
		return( ERROR );
	(void) msgClose( mHandle );
	return( OK );
}
	


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpSend
 *
 * Arguments:	mType, specifies messagetype (IN).
 *
 * Purpose:	
 *
 * Remarks:
 *
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpSend( IADDRESS remoteAddress, MSG *mMsg )
{
	register short i;
	int s;
	char m[32*1024];

	s = sizeof( MSG );
	memcpy( m+4, mMsg, s );
	memcpy( m, &s, 4 );

	if( simSend( (char *) remoteAddress, (char *) m ) != OK ) {
		for( i = msgCacheIndex+1; i != msgCacheIndex; i++ ) {
			if( msgCache[i].type == DEFAULTVALUE ) {
				memcpy( &msgCache[i], mMsg, sizeof( MSG ) );
				return( OK );
			}
			if( i >= MSGTABLEMAX ) 
				i = -1;
		}
		return( FAILED );
	}
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpSendRequest
 *
 * Arguments:	mType, specifies messagetype (IN).
 *		mSysName, specifies systemname (IN).
 *		mReqId, identifies message (IN).
 *		mQos, specifies the quality of service (IN).
 *		mErrInd, indexes erroneous variable (IN)
 *		mErrStat, identifies errortype (IN).
 *		mConName, specifies the contextname (IN).
 *		mVarBinds, reference to variablebindingslist (IN).
 *
 * Purpose: 	This function sends the mType message to the SPM.
 *		Steps:
 *		1) Allocate messagebuffer from bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Fill messagebuffer.
 *			ERROR ->	Function returns ERROR.
 *		3) Send message.
 *			ERROR ->	Function returns ERROR.
 *		4) Release messagebuffer.
 *			ERROR ->	Function returns ERROR.
 *		5) Function returns OK
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpSendRequest( SPTYPE mType, char *mSysName, INTEGER mReqId, QOS mQos,
		 INTEGER mErrInd, INTEGER mErrStat, char *mConName,
		 VARLIST *mVarBinds )
{
	AsnInt mHandle;
	MSG *mMsg;

	if( msgOpen( &mHandle, &mMsg ) != OK )
		return( FAILED );
	if( msgPut( mType, mSysName, mQos, mConName, mReqId, mVarBinds, 
		    mErrStat, mErrInd, mHandle ) != OK )
		return( FAILED );
	if( snmpSend( spmAddress, mMsg ) != OK )
		return( FAILED );
	(void) msgClose( mHandle );
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	snmpSendResponse
 *
 * Arguments:	mType, specifies messagetype (IN).
 *		mReqId, identifies message (IN).
 *		mErrInd, indexes erroneous variable (IN)
 *		mErrStat, identifies errortype (IN).
 *		mVarBinds, reference to variablebindingslist (IN).
 *
 * Purpose: 	This function sends the mType message to the SPM.
 *		Steps:
 *		1) Allocate messagebuffer from bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Fill messagebuffer.
 *			ERROR ->	Function returns ERROR.
 *		3) Send message.
 *			ERROR ->	Function returns ERROR.
 *		4) Release messagebuffer.
 *			ERROR ->	Function returns ERROR.
 *		5) Function returns OK
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
snmpSendResponse( SPTYPE mType, INTEGER mReqId, INTEGER mErrStat, 
		  INTEGER mErrInd, VARLIST *mVarBinds )
{
	AsnInt mHandle;
	MSG *mMsg;

	if( msgOpen( &mHandle, &mMsg ) != OK )
		return( ERROR );
	if( msgPut( mType, NULL, (QOS) NULL, NULL, mReqId, mVarBinds, mErrStat, 
		    mErrInd, mHandle ) != OK )
		return( ERROR );
	if( snmpSend( spmAddress, mMsg ) != OK )
		return( ERROR );
	(void) msgClose( mHandle );
	return( OK );
}



/* -----------------------------------------------------------------------
 */
