/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename: 	TCP.c
 *
 * Date:		July 1994
 *
 * Purpose:		implementation of transport level interface with sockets
 *
 * Author(s):	Vincent Berkhout, Frederic P. H. van Hengstum 
 *
 * -----------------------------------------------------------------------
 */

#include <tcp.h>

/*
 * 	local variables
 */
static int SNMP_DEBUG_SOCKET = 31;

/*------------------------------------------------------------------------*/
	STATUS
tcpPassiveOpen( localTAddress,  connectSocket) 
u_char 	*localTAddress ;	/* in, identifies the local port to listen on */
int	*connectSocket ;	/* out, return the "listen" filedescriptors */
/*
 * Purpose:
 * 	This call performs a passive open for the server side. It derives the
 *  	host and portnumber from the localTAddress parameter. It subsequently creates
 *	a socket, binds it to the portnumber and goes into "listen" mode.
 * Remarks:
 */
{
	long	hostNr ;
	short	portNr ;
	struct 	sockaddr_in	newSock;
/* 
 * 	first  open a socket with some initial values ( tcp state is initialy CLOSED)
 */
	*connectSocket = socket( PF_INET,  SOCK_STREAM, 0 ) ;
	if( *connectSocket < 0)   {
		if (SNMP_DEBUG_SOCKET)
		perror( "spmSocket'SocketStreamPassiveOpensocket'socket" ) ;
		return(ERROR) ;
	}	/* endif socket creation */
/*
 *	Note that no socket packet size is set, because this is only a listen socket
 *
 * 	convert the local address, only the port number is relevant
 */
  	(void) memcpy( &(hostNr), (const void *) localTAddress,  4 );
	(void) memcpy( &(portNr), (const void *) (localTAddress + 4) , 2);
/*
 *	fill in the internet address and port in the address structure newSock
 */
	newSock.sin_family = AF_INET ;
	newSock.sin_addr.s_addr = INADDR_ANY ;
	newSock.sin_port = htons(portNr) ;
/* 
 * 	now bind the socket to the transport  address  which was just filled in
 */
	if (bind( *connectSocket , (struct sockaddr *)&newSock, sizeof(newSock) ) < 0 ) {
		/* TRACE( SNMP_DEBUG_SOCKET, "address already in use, bind failed") ; */
		(void) close (*connectSocket) ;
		return ERROR ;
	}	
/* 
 *	The binding has succeeded, now listen for new connections on the socket
 * 	maximum number of request in queue is 5 
 */
	if (listen( *connectSocket , 5) < 0) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketStreamPassiveOpensocket'listen") ;
		return ERROR ;
	}	/* endif listen */ 
	return OK ;

} 	/* end of function SocketStreamPassiveOpen */


/*------------------------------------------------------------------------*/
	STATUS
tcpActiveOpen( remoteTAddress, dataSocket)
u_char	*remoteTAddress; /* in, the transport address of the remote machine */
int	*dataSocket;	/* out, filedescriptor, ready for sending and receiveving */
/* 
 * Purpose:
 * 	This call performs an active open at the client size. It derives the IP number
 * 	and portnumber from the tranport address. First it creates a soocket and then 
 * 	calls connect
 * Remarks: 
 */
{
	long	hostNr;
	short	portNr;
	int	msgSize = PACKETSIZE ;
	struct	sockaddr_in 	destination;
/* 
 * 	first create the data socket 
 */
	*dataSocket = socket( PF_INET, SOCK_STREAM, 0 );
	if( *dataSocket < 0)   {
		if (SNMP_DEBUG_SOCKET)
		perror( "spmSocket'SocketStreamActiveOpen'socket") ;
		return ERROR  ;
	}	/* endif socket creation */

/*
 *	set the packetsize as big as possible, preferably at 64 K
 */ 
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_SNDBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketStreamActiveOpen'setsockoptSNDBUF") ;
	} 	/* endif set SNDBUF */
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_RCVBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketStreamActiveOpen'setsockoptRCVBUF") ;
	} 	/* endif set RCVBUF */

/* 
 * 	convert out the remote address 
 */
	(void) memset( (char *)&destination, '\0', sizeof(struct sockaddr_in)) ; 
  	(void) memcpy( &(hostNr), (const void *)remoteTAddress,  4 );
	(void) memcpy( &(portNr), (const void *)(remoteTAddress + 4) , 2);
/*
 *	fill in the destination internet address (struct )
 */
	destination.sin_family = AF_INET;   /* tcp belongs to the internet address family*/
	destination.sin_addr.s_addr =  htonl(hostNr) ;
	destination.sin_port = htons(portNr) ;
/* 
 * 	now connect the socket with remote destination address 
 */
	if ( connect (*dataSocket, (struct sockaddr *)&destination, sizeof( destination)  ) < 0) {
		if (SNMP_DEBUG_SOCKET)
		perror("spmSocket'SocketStreamActiveOpen'connect") ;
		(void) close (*dataSocket ) ;
		return FAILED ;
	} 	/* endif connect */

	return OK ;

}	/* end of function SocketStreamActiveOpen */


/*------------------------------------------------------------------------*/
 	STATUS	
tcpAcceptOpen( connectSocket, dataSocket, remoteTAddress) 
int 	*connectSocket, 	/* in, the connectSocket that is active */
	*dataSocket ;		/* out, the new dataSocket with connection */
u_char	*remoteTAddress;	/* out, the remote address of the peer */
/* 
 * Purpose:
 * 	This function accepts a data indication. When the server is in the listen state
 * 	and a client performs an active open, this function is called to accept the new 
 * 	connection (at the server side!)
 * Remarks:
 */
{
	struct	sockaddr_in	remote;
	int	remoteAddressLen,
		msgSize = PACKETSIZE ;
	long	hostNr ;
	short	portNr ;

	remoteAddressLen = sizeof( remote ) ;
	*dataSocket = accept( *connectSocket, (struct sockaddr *)&remote, &remoteAddressLen) ;
/*
 *	set the packetsize as big as possible, preferably at 64 K
 */ 
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_SNDBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
	} 	/* endif set SNDBUF */
	if ( setsockopt(*dataSocket, SOL_SOCKET, SO_RCVBUF, (char *)&msgSize, sizeof(msgSize) ) < 0 ) {
	} 	/* endif set RCVBUF */
/* 
 * 	if correct then remoteTAddress now contains the other sides socket address
 */
	if ( *dataSocket < 0) {
		return FAILED ;
	} else {
		hostNr = ntohl(remote.sin_addr.s_addr) ;
		portNr = ntohs(remote.sin_port) ;
	 	(void) memcpy( (void *) remoteTAddress, (const void *)&hostNr,   4 );
		(void) memcpy( (void *) (remoteTAddress + 4), (const void *)&portNr,  2);
		return OK ;
	}	/* endif datasocket */
}	/* end of function SocketStreamAcceptConnection */


/*------------------------------------------------------------------------*/
	STATUS	
tcpSend( dataSocket, message)
int	*dataSocket;	/* in, the socket where to the data is send */
char	*message;	/* in, the actual data (or message)  */
/*
 * Purpose:
 * 	This is the function where the data is send, using the syscall "send"	
 * Remarks:
 * 	There is still an error with the maximum sendable (?) size.....
 */
{
	int	nrWritten,
		messageSize;

	(void) memcpy( &messageSize, message, 4 );
	nrWritten = write( *dataSocket, message, messageSize+4 );

	if ( nrWritten <= 0) {
		return FAILED ;
	}	/* endif nrWritten */
	return OK ;
} 	/* end of function SocketStreamSend */


/*------------------------------------------------------------------------*/
	STATUS
tcpReceive( dataSocket, message )
int	*dataSocket ;	/* in, the socket that is active  */
u_char	*message ;	/* out, the data that has been read, if any */
/* 
 * Purpose:
 * 	Reads data from a given socket and returns it as a message.
 * Remarks:
 */
{
	int 	nrReads,
		messageSize ;

	messageSize = PACKETSIZE ;

	nrReads = read (*dataSocket, message, messageSize ) ;

	if (nrReads <= 0) {
		return ERROR ;
	}	/* endif nrReads */
	return OK ;
}	/* end of function SocketStreamReceive */


/*---------------------- end of file spmSocket.c -----------------------------*/

