/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    simCL.c
 *
 * Date:        July 1994
 *
 * Purpose:		This file maps abstract data types upon real data types 
 *				and distributes active to the appropriate
 *				underlying interprocess communication domain.
 *
 * Author(s);   Vincent Berkhout, Frederic van Hengstum, Rob Post 
 *
 * -----------------------------------------------------------------------
 */

#include <sim.h>


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    address
 *
 * Arguments:   portNr, desired port number (IN)
 *				a, reference to local endpoint (IN/OUT)
 *
 * Returns:		returns internet address of endpoint.
 *
 * Purpose:		Create address of endpoint
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
char *address( unsigned short portNr, char *a )
{
    int hostNr;

    hostNr = ((struct in_addr *)((gethostbyname(getenv("HOST")))->h_addr))->s_addr;
    memcpy( a, (const void *) &hostNr, 4 );
    memcpy( a+4, (const void *) &portNr, 2 );
	return a;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfCheckInputs
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				allHandles, all handles that might be active (IN)
 *				activeHandles, all active handles (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfCheckInputs( IDOMAIN domain, IHANDLE *allHandles, IHANDLE *activeHandles ) 
{
	switch( domain )
	{
		case UDP_DOMAIN:
		case TCP_DOMAIN:
			return( udpCheckInputs( (int *)allHandles, (int *) activeHandles ) ); 
		case SHM_DOMAIN :
			return( shmCheckInputs( (int *) allHandles, (int *) activeHandles ) );
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfClose
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				handle, handle to close (IN)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfClose( IDOMAIN domain, IHANDLE handle)
{
	switch( domain )
	{
		case UDP_DOMAIN:
		case TCP_DOMAIN:
			return( udpClose( (int) handle ) );
		case SHM_DOMAIN :
			return( shmClose( (int) handle ) );
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfPassiveOpen
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				localTAddress, local address containing the portnumber (IN)
 *				handle, a listen handle for new connections (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfPassiveOpen( IDOMAIN domain, IADDRESS localAddress, IHANDLE *handle ) 
{
	char a[6];

	switch( domain )
	{
		case TCP_DOMAIN:
			return( tcpPassiveOpen( address(8160,a), handle ) ); 
		case UDP_DOMAIN:
			return( udpOpen( address(54321,a), handle ) );
		case SHM_DOMAIN :
			return( shmPassiveOpen( localAddress, handle ) );
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfActiveOpen
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				address, address of remote entity (IN)
 *				handle, the handle of the new connection (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfActiveOpen( IDOMAIN domain, IADDRESS l, IADDRESS r, IHANDLE *handle )  
{
	char a[6];

	switch( domain )
	{
		case TCP_DOMAIN:
			return( tcpActiveOpen( address(8160,a), handle ) ); 
		case UDP_DOMAIN:
			return( udpOpen( address(54321,a), handle ) ); 
		case SHM_DOMAIN :
 			return( shmActiveOpen( l, r, handle ) );
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfAcceptOpen
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				connectHandle, the listen handle (IN)
 *				address, the address of the remote peer (OUT)
 *				handle, the handle referencing to new connection (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfAcceptOpen( IDOMAIN domain, IHANDLE *connectHandle, 
			   IADDRESS address, IHANDLE *handle )  
{
	switch( domain )
	{
		case TCP_DOMAIN:
			return( tcpAcceptOpen( (int *) connectHandle,
							(int *) handle, (const char*) address) ); 
		case SHM_DOMAIN:
			return( shmAcceptOpen( connectHandle, address, handle ) );
	}
	return FAILED ;
}
 

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfSend
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				handle, data handle on which to send the msg (IN)
 *				message, the msg to be send (IN)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfSend( IDOMAIN domain, IHANDLE handle, IMESSAGE message )  
{
	char a[6];

	switch( domain )
	{
		case TCP_DOMAIN:
			return( tcpSend( (int *) &handle, message ) ); 
		case UDP_DOMAIN:
			return( udpSend( &handle, address(handle,a), message ) ); 
		case SHM_DOMAIN :
			return( shmSend( (int *) handle, message ) ); 
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    lpfReceive
 *
 * Arguments:   domain, type of ipc domain (IN)
 *				handle, data handle on which to receive the msg (IN)
 *				msg, the msg that has been received (OUT)
 *
 * Returns:		Transparantly returns the underlying return value.
 *
 * Purpose:		Convert abstract functionality to real functionality and
 *      		abstract data types to real data types.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
lpfReceive( IDOMAIN domain, IHANDLE *handle, IMESSAGE msg )  
{
	switch( domain )
	{
		case TCP_DOMAIN:
			return( tcpReceive( (int *) handle, (char *) msg ) ); 
		case UDP_DOMAIN:
			return( udpReceive( (int *) handle, (char *) msg ) ); 
		case SHM_DOMAIN:
			return( shmReceive( *handle, msg ) );
	}
	return FAILED ;
}
 

/* -----------------------------------------------------------------------
 */
